keycode   8 = Cancel NoSymbol F11
keycode   9 = SunAudioLowerVolume SunVideoLowerBrightness
keycode  10 = Redo NoSymbol F12
keycode  11 = SunAudioRaiseVolume SunVideoRaiseBrightness
keycode  12 = F1
keycode  13 = F2
keycode  14 = F10
keycode  15 = F3
keycode  16 = F11 NoSymbol SunF36
keycode  17 = F4
keycode  18 = F12 NoSymbol SunF37
keycode  19 = F5
keycode  20 = Mode_switch
keycode  21 = F6
keycode  22 =
keycode  23 = F7
keycode  24 = F8
keycode  25 = F9
keycode  26 = Alt_L
keycode  27 = Up
keycode  28 = Pause NoSymbol Break
keycode  29 = Print NoSymbol Sys_Req SunSys_Req
keycode  30 = Scroll_Lock
keycode  31 = Left
keycode  32 = SunProps NoSymbol F13
keycode  33 = Undo NoSymbol F14
keycode  34 = Down
keycode  35 = Right
keycode  36 = Escape
keycode  37 = 1 exclam
keycode  38 = 2 at
keycode  39 = 3 numbersign
keycode  40 = 4 dollar
keycode  41 = 5 percent
keycode  42 = 6 asciicircum
keycode  43 = 7 ampersand
keycode  44 = 8 asterisk
keycode  45 = 9 parenleft
keycode  46 = 0 parenright
keycode  47 = minus underscore
keycode  48 = equal plus
keycode  49 = grave asciitilde acute
keycode  50 = BackSpace
keycode  51 = Insert
keycode  52 = SunAudioMute SunVideoDegauss
keycode  53 = KP_Divide NoSymbol F25
keycode  54 = KP_Multiply NoSymbol F26
keycode  55 = SunPowerSwitch SunPowerSwitchShift
keycode  56 = SunFront NoSymbol F15
keycode  57 = KP_Delete KP_Decimal
keycode  58 = SunCopy NoSymbol F16
keycode  59 = Home
keycode  60 = Tab ISO_Left_Tab
keycode  61 = q Q
keycode  62 = w W
keycode  63 = e E eogonek Eogonek
keycode  64 = r R
keycode  65 = t T
keycode  66 = y Y
keycode  67 = u U
keycode  68 = i I
keycode  69 = o O oacute Oacute
keycode  70 = p P
keycode  71 = bracketleft braceleft
keycode  72 = bracketright braceright
keycode  73 = Delete
keycode  74 = Multi_key
keycode  75 = KP_Home KP_7 F27
keycode  76 = KP_Up KP_8 F28
keycode  77 = KP_Prior KP_9 F29
keycode  78 = KP_Subtract
keycode  79 = SunOpen NoSymbol F17
keycode  80 = SunPaste NoSymbol F18
keycode  81 = End
keycode  82 =
keycode  83 = Control_L
keycode  84 = a A aogonek Aogonek
keycode  85 = s S sacute Sacute
keycode  86 = d D
keycode  87 = f F
keycode  88 = g G
keycode  89 = h H
keycode  90 = j J
keycode  91 = k K
keycode  92 = l L lstroke Lstroke
keycode  93 = semicolon colon
keycode  94 = apostrophe quotedbl acute
keycode  95 = backslash bar
keycode  96 = Return
keycode  97 = KP_Enter
keycode  98 = KP_Left KP_4 F30
keycode  99 = KP_Begin KP_5 F31
keycode 100 = KP_Right KP_6 F32
keycode 101 = KP_Insert KP_0
keycode 102 = Find NoSymbol F19
keycode 103 = Prior
keycode 104 = SunCut NoSymbol F20
keycode 105 = Num_Lock
keycode 106 = Shift_L
keycode 107 = z Z zabovedot Zabovedot
keycode 108 = x X zacute Zacute
keycode 109 = c C cacute Cacute
keycode 110 = v V
keycode 111 = b B
keycode 112 = n N nacute Nacute
keycode 113 = m M
keycode 114 = comma less
keycode 115 = period greater
keycode 116 = slash question
keycode 117 = Shift_R
keycode 118 =
keycode 119 = KP_End KP_1 F33
keycode 120 = KP_Down KP_2 F34
keycode 121 = KP_Next KP_3 F35
keycode 122 =
keycode 123 =
keycode 124 =
keycode 125 = Help
keycode 126 = Caps_Lock
keycode 127 = Meta_L
keycode 128 = space
keycode 129 = Meta_R
keycode 130 = Next
keycode 131 =
keycode 132 = KP_Add
