#ifndef __GWEATHER_ABOUT_H_
#define __GWEATHER_ABOUT_H_

/* $Id$ */

/*
 *  Papadimitriou Spiros <spapadim+@cs.cmu.edu>
 *
 *  This code released under the GNU GPL.
 *  Read the file COPYING for more information.
 *
 *  About box
 *
 */

#define GWEATHER_I_KNOW_THIS_IS_UNSTABLE

#include "gweather.h"

G_BEGIN_DECLS

extern void gweather_about_run (GWeatherApplet *gw_applet);

G_END_DECLS

#endif /* __GWEATHER_ABOUT_H_ */

